@@ -20,9 +20,12 @@ if os.environ.has_key("DISPLAY"):
 			pass
 	import gtk
 
-
 from bottlelib import BottleClient, BottleClientError
-import SakuraScript
+
+try:
+	from ninix.script import *
+except:
+	from script import *
 
 gettext.install('gbottler')
 
@@ -32,6 +35,9 @@ LIBDIR = "~/.bottlecase"
 VOTE  = "Vote"
 AGREE = "Agree"
 
+APP = 'GBottler'
+VER = '0.0.3'
+
 class Bottler(BottleClient):
 	def __init__(self, app):
 		BottleClient.__init__(self, app.pbar)
@@ -119,6 +125,18 @@ class Application:
 	def edit_delete(self, data, widget):
 		self.main_text.emit("delete-from-cursor", gtk.DELETE_CHARS, 1)
 
+	def get_imglabelbox_with_stock(self, stock_id, size, label):
+		h = gtk.HBox(gtk.FALSE, 0)
+		i = gtk.Image()
+		i.set_from_stock(stock_id, size)
+		l = gtk.Label(label)
+		h.pack_start(i, gtk.FALSE, gtk.TRUE, 0)
+		h.pack_start(l, gtk.FALSE, gtk.TRUE, 0)
+		i.show()
+		l.show()
+		h.show()
+		return h
+
 	DEFAULT_TARGET = unicode(_("default"), "utf-8")
 	def __init__(self):
 		self.msg = {
@@ -189,8 +207,7 @@ class Application:
 		self.window.connect("destroy", self.quit, "WM destroy")
 		self.window.set_title(unicode(_("GBottler"), "utf-8"))
 
-		self.main_vbox = gtk.VBox(gtk.FALSE, 1)
-		self.main_vbox.set_border_width(1)
+		self.main_vbox = gtk.VBox(gtk.FALSE, 0)
 		self.window.add(self.main_vbox)
 		self.main_vbox.show()
 
@@ -222,22 +239,50 @@ class Application:
 		self.menubar.show()
 		# End of creating Menubar
 
-		checkButton = gtk.Button(unicode(_("Check"), "utf-8"))
+		check_h = self.get_imglabelbox_with_stock(gtk.STOCK_SPELL_CHECK,
+												  gtk.ICON_SIZE_BUTTON,
+												  unicode(_("Check"), "utf-8"))
+		checkButton = gtk.Button()
+		checkButton.add(check_h)
 		checkButton.connect("clicked", self.check, None)
-		testButton  = gtk.Button(unicode(_("Test"), "utf-8"))
+		check_h.show()
+
+		test_h = self.get_imglabelbox_with_stock(gtk.STOCK_REFRESH,
+												 gtk.ICON_SIZE_BUTTON,
+												 unicode(_("Test"), "utf-8"))
+		testButton = gtk.Button()
+		testButton.add(test_h)
 		testButton.connect("clicked", self.test, None)
-		self.send_button  = gtk.Button(unicode(_("Send"), "utf-8"))
+		test_h.show()
+
+		send_h =  self.get_imglabelbox_with_stock(gtk.STOCK_OK,
+												  gtk.ICON_SIZE_BUTTON,
+												  unicode(_("Send"), "utf-8"))
+		self.send_button = gtk.Button()
+		self.send_button.add(send_h)
 		self.send_button.connect("clicked", self.send, None)
+		send_h.show()
 
 		channelLabel	   = gtk.Label(unicode(_("Channel") + ": ", "utf-8"))
+
 		self.channel	   = gtk.Label()
+		self.c_e_box       = gtk.EventBox()
+		self.c_e_box.set_events(gtk.gdk.BUTTON_PRESS_MASK)
+		self.c_e_box.add(self.channel)
+		self.c_e_box.show()
+
 		self.channel_ghost = gtk.Label()
+		self.g_e_box       = gtk.EventBox()
+		self.g_e_box.set_events(gtk.gdk.BUTTON_PRESS_MASK)
+		self.g_e_box.add(self.channel_ghost)
+		self.g_e_box.show()
+
 		usersLabel		 = gtk.Label("   " + unicode(_("Users") + ": ", "utf-8"))
 		self.users		 = gtk.Label()
 
-		group2 = gtk.HBox(gtk.FALSE, 1)
+		group2 = gtk.HBox(gtk.FALSE, 0)
 		group2.set_border_width(1)
-		self.main_vbox.add(group2)
+		self.main_vbox.pack_start(group2, gtk.FALSE, gtk.TRUE, 0)
 		group2.show()
 
 		group2.pack_start(checkButton, gtk.FALSE, gtk.TRUE, 0)
@@ -245,8 +290,8 @@ class Application:
 		group2.pack_start(self.send_button, gtk.FALSE, gtk.TRUE, 0)
 
 		group2.pack_start(channelLabel, gtk.FALSE, gtk.TRUE, 0)
-		group2.pack_start(self.channel, gtk.FALSE, gtk.TRUE, 0)
-		group2.pack_start(self.channel_ghost, gtk.FALSE, gtk.TRUE, 0)
+		group2.pack_start(self.c_e_box, gtk.FALSE, gtk.TRUE, 0)
+		group2.pack_start(self.g_e_box, gtk.FALSE, gtk.TRUE, 0)
 		group2.pack_start(usersLabel, gtk.FALSE, gtk.TRUE, 0)
 		group2.pack_start(self.users, gtk.FALSE, gtk.TRUE, 0)
 
@@ -272,11 +317,22 @@ class Application:
 		self.main_text.set_wrap_mode(gtk.WRAP_CHAR)
 		sw.add(self.main_text)
 		sw.show()
-		self.main_vbox.add(sw)
+		#self.main_vbox.add(sw)
 		self.main_text.show()
 		self.main_text.grab_focus()
 		self.main_text.emit("move-cursor", gtk.MOVEMENT_LOGICAL_POSITIONS, -2, gtk.FALSE)
 
+		self.pbar = gtk.ProgressBar()
+		self.pbar.set_size_request(500, 20)
+		#self.main_vbox.pack_start(self.pbar, gtk.FALSE, gtk.TRUE, 0)
+		self.pbar.show()
+
+		self.paned = gtk.VPaned()
+		self.paned.pack1(sw)
+		self.paned.pack2(self.pbar)
+		self.main_vbox.pack_start(self.paned, gtk.TRUE, gtk.TRUE, 0)
+		self.paned.show()
+
 		pmsg = {
 			"sessions":     unicode(_("Sessions"), "utf-8"),
 			"timecritical": unicode(_("TimeCritical"), "utf-8"),
@@ -330,15 +386,15 @@ class Application:
 
 			( "/%s" % pmsg["sakura"], None, self.insert_h, 0, None ),
 			( "/%s" % pmsg["countenance"], None, None, 0, "<Branch>" ),
-			( "/%s/[0]  %s" % (pmsg["countenance"], pmsg["general"]), "<control><alt>g", self.insert_s0,  0, None, ),
-			( "/%s/[1]  %s" % (pmsg["countenance"], pmsg["shy"]), None, self.insert_s1, 0, None, ),
-			( "/%s/[2]  %s" % (pmsg["countenance"], pmsg["surprised"]), None, self.insert_s2, 0, None, ),
-			( "/%s/[3]  %s" % (pmsg["countenance"], pmsg["angst"]), None, self.insert_s3, 0, None, ),
-			( "/%s/[4]  %s" % (pmsg["countenance"], pmsg["ease off"]), None, self.insert_s4, 0, None, ),
-			( "/%s/[5]  %s" % (pmsg["countenance"], pmsg["smile"]), None, self.insert_s5, 0, None, ),
-			( "/%s/[6]  %s" % (pmsg["countenance"], pmsg["closed eyes"]), None, self.insert_s6, 0, None, ),
-			( "/%s/[7]  %s" % (pmsg["countenance"], pmsg["angry"]), None, self.insert_s7, 0, None, ),
-			( "/%s/[8]  %s" % (pmsg["countenance"], pmsg["scorned"]), None, self.insert_s8, 0, None, ),
+			( "/%s/[0]  %s" % (pmsg["countenance"], pmsg["general"]), "<control><alt>0", self.insert_s0,  0, None, ),
+			( "/%s/[1]  %s" % (pmsg["countenance"], pmsg["shy"]), "<control><alt>1", self.insert_s1, 0, None, ),
+			( "/%s/[2]  %s" % (pmsg["countenance"], pmsg["surprised"]), "<control><alt>2", self.insert_s2, 0, None, ),
+			( "/%s/[3]  %s" % (pmsg["countenance"], pmsg["angst"]), "<control><alt>3", self.insert_s3, 0, None, ),
+			( "/%s/[4]  %s" % (pmsg["countenance"], pmsg["ease off"]), "<control><alt>4", self.insert_s4, 0, None, ),
+			( "/%s/[5]  %s" % (pmsg["countenance"], pmsg["smile"]), "<control><alt>5", self.insert_s5, 0, None, ),
+			( "/%s/[6]  %s" % (pmsg["countenance"], pmsg["closed eyes"]), "<control><alt>6", self.insert_s6, 0, None, ),
+			( "/%s/[7]  %s" % (pmsg["countenance"], pmsg["angry"]), "<control><alt>7", self.insert_s7, 0, None, ),
+			( "/%s/[8]  %s" % (pmsg["countenance"], pmsg["scorned"]), "<control><alt>8", self.insert_s8, 0, None, ),
 			( "/%s/[20] %s" % (pmsg["countenance"], pmsg["signboard"]), None, self.insert_s20, 0, None, ),
 			( "/%s/[25] %s" % (pmsg["countenance"], pmsg["singing"]), None, self.insert_s25, 0, None, ),
 			( "/sep2", None, None, 0, "<Separator>" ),
@@ -398,16 +454,12 @@ class Application:
 		self.popup_menu = self.popup_item_factory.get_widget("<main>")
 		self.main_text.connect("button-press-event", self.popup)
 
-		self.pbar = gtk.ProgressBar()
-		self.main_vbox.add(self.pbar)
-		self.pbar.show()
-
 		self.window.show()
 
 		self.client = Bottler(self)
 		self.client.debug = 2
 
-		self.parser = SakuraScript.Parser()
+		self.parser = Parser('loose')
 
 		self.host = ""
 		self.port = 9801
@@ -444,7 +496,8 @@ class Application:
 			d.run()
 			d.destroy()
 
-		self.menu_list["/%s/%s" % (self.msg["window"][1], self.msg["logs"])].connect("activate", self.log_window.open, None)
+		self.menu_list["/%s/%s" % (self.msg["window"][1],
+								   self.msg["logs"])].connect("activate", self.log_window.open, None)
 
 		self.about = None
 
@@ -452,6 +505,8 @@ class Application:
 		self.local_ghost_last_notified = 0
 		self.send_ghost_names()
 		# initialize
+		self.c_e_id = 1 # channel label eventbox handler id
+		self.g_e_id = 1
 		self.disable_channel()
 
 		self.timeout_id = None
@@ -492,6 +547,10 @@ class Application:
 		self.enable_channel()
 		self.timeout_id = gtk.timeout_add(100, self.handle_event)
 
+	def button_popup(self, widget, event, sender):
+		if event.button in [1, 3]:
+			sender.popup(None, None, None, event.button, 0)
+
 	def enable_channel(self):
 		# rebuild Channel menu
 		channel_item = self.menu_list["/%s" % self.msg["channel"][1]]
@@ -525,6 +584,10 @@ class Application:
 		channel_item.set_submenu(self.channel_factory.get_widget("<main>"))
 		channel_item.set_sensitive(gtk.TRUE)
 
+		if not self.c_e_box.handler_is_connected(self.c_e_id):
+			self.c_e_id = self.c_e_box.connect("button-press-event", self.button_popup,
+											   self.channel_factory.get_widget("<main>"))
+
 		self.rebuild_ghostmenu()
 		self.menu_list["/%s/%s" % (self.msg["file"][1], self.msg["part"])].set_sensitive(gtk.TRUE)
 		self.menu_list["/%s/%s" % (self.msg["file"][1], self.msg["join"])].set_sensitive(gtk.FALSE)
@@ -544,6 +607,9 @@ class Application:
 		ghost_item.deselect()
 		ghost_item.set_sensitive(gtk.FALSE)
 
+		if self.g_e_box.handler_is_connected(self.g_e_id):
+			self.g_e_box.disconnect(self.g_e_id)
+
 		current = self.target_ghost
 		ghosts = [self.DEFAULT_TARGET]
 		ghosts.append(self.sakuraname)
@@ -583,6 +649,9 @@ class Application:
 		ghost_item.set_submenu(self.g_ifact.get_widget("<main>"))
 		ghost_item.set_sensitive(gtk.TRUE)
 
+		if not self.g_e_box.handler_is_connected(self.g_e_id):
+			self.g_e_id = self.g_e_box.connect("button-press-event", self.button_popup, self.g_ifact.get_widget("<main>"))
+
 		self.check_channel_permission()
 
 	def update_ghost(self, widget, data):
@@ -667,6 +736,13 @@ class Application:
 		self.log_window.vi.set_sensitive(gtk.FALSE)
 		self.log_window.ai.set_sensitive(gtk.FALSE)
 
+		if self.c_e_box.handler_is_connected(self.c_e_id):
+			self.c_e_box.disconnect(self.c_e_id)
+			self.c_e_id = 1
+		if self.g_e_box.handler_is_connected(self.g_e_id):
+			self.g_e_box.disconnect(self.g_e_id)
+			self.g_e_id = 1
+
 		self.send_button.set_sensitive(gtk.FALSE)
 		self.channel.set_text("")
 		self.channel_ghost.set_text("")
@@ -745,7 +821,7 @@ class Application:
 	def check_line(self, script, lineno):
 		try:
 			return self.parser.parse(script)
-		except SakuraScript.ParserError, e:
+		except ParserError, e:
 			self.pbar.set_fraction(0.0)
 			self.monitor_insert(unicode(_("Error"), "utf8"))
 
@@ -754,23 +830,40 @@ class Application:
 			d.connect("destroy", lambda w,data:d.destroy(), "WM destroy")
 			d.vbox.set_border_width(10)
 
+			h = gtk.HBox()
+			i = gtk.Image()
+			i.set_from_stock(gtk.STOCK_DIALOG_ERROR, gtk.ICON_SIZE_LARGE_TOOLBAR)
+			i.show()
+			h.pack_start(i, gtk.FALSE, gtk.TRUE, 0)
+			l = gtk.Label("Script Parser Error")
+			l.show()
+			h.pack_start(l, gtk.FALSE, gtk.TRUE, 0)
+			h.show()
+			d.vbox.add(h)
+			s = gtk.HSeparator()
+			s.show()
+			d.vbox.add(s)
+
 			s = gtk.ScrolledWindow()
 			s.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
 			t = gtk.TextView()
 			t.set_cursor_visible(gtk.TRUE)
 			t.set_size_request(300, 100)
 			b = t.get_buffer()
+			b.set_text('')
+			tag = b.create_tag()
+			tag.set_property('font', 'Monospace')
 
-			b.set_text(unicode(_("Line"), "utf8") + \
-					   " %d: %s\n%s\n" % (lineno, e.message, script) +
-					   " " * e.column + "^" * (e.length or 1))
+			b.insert_with_tags(b.get_end_iter(), unicode(_("Line"), "utf8") + \
+							   " %d: %s\n%s\n" % (lineno, e.message, script) +
+							   " " * e.column + "^" * (e.length or 1), tag)
 
 			t.set_editable(gtk.FALSE)
 			t.set_wrap_mode(gtk.WRAP_CHAR)
 			s.add(t)
 			d.vbox.add(s)
 
-			ok = gtk.Button(_("Close"))
+			ok = gtk.Button(_("Close"), gtk.STOCK_CLOSE)
 			ok.connect("clicked", lambda w,data:d.destroy(), None)
 			ok.show()
 			d.action_area.pack_start(ok, gtk.TRUE, gtk.TRUE, 0)
@@ -790,14 +883,18 @@ class Application:
 		msg_or  = unicode(_("or"), "utf8")
 		msg_tag = unicode(_("tag"), "utf8")
 		for node in script:
-			if node[0] == SakuraScript.SCRIPT_TAG:
+			if node[0] == SCRIPT_TAG:
 				name, args = node[1], node[2:]
 				if name == r"\h":
 					scope = 0
 				elif name == r"\u":
 					scope = 1
 				elif name == r"\s":
-					id = int(args[0])
+					try:
+						id = int(args[0])
+					except ValueError:
+						error = unicode(_('bad use of surfaces'), 'utf-8')
+						break
 					if id in self.RESERVED_SURFACES:
 						target = id / 10 % 2
 					else:
@@ -815,7 +912,7 @@ class Application:
 						break
 					if id in self.RESERVED_SURFACES:
 						surface_usage[scope] = id
-			elif node[0] == SakuraScript.SCRIPT_TEXT:
+			elif node[0] == SCRIPT_TEXT:
 				if scope is None:
 					error = r"\h %s \u " % msg_or + \
 							unicode(_("is needed before characters"), "utf8")
@@ -825,6 +922,7 @@ class Application:
 			   surface_usage[1] / 10 % 2 == 0:
 				error = unicode(_("bad use of surfaces"), "utf8")
 		if error is not None:
+			self.pbar.set_fraction(0.0)
 			self.monitor_clear()
 			self.monitor_insert(unicode(_("Error"), "utf8") + ": %s " % error)
 			return 1
@@ -904,7 +1002,21 @@ class Application:
 		self.about.connect("destroy", aboutdestroy, "WM destroy")
 		self.about.set_border_width(10)
 
-		frame = gtk.Frame(label=" GBottler 0.0.1 ")
+		h = gtk.HBox()
+		i = gtk.Image()
+		i.set_from_stock(gtk.STOCK_DIALOG_INFO, gtk.ICON_SIZE_LARGE_TOOLBAR)
+		i.show()
+		h.pack_start(i, gtk.FALSE, gtk.TRUE, 0)
+		l = gtk.Label(unicode(_("About"), 'utf-8'))
+		l.show()
+		h.pack_start(l, gtk.FALSE, gtk.TRUE, 0)
+		h.show()
+		self.about.vbox.add(h)
+		s = gtk.HSeparator()
+		s.show()
+		self.about.vbox.add(s)
+
+		frame = gtk.Frame(label=" %s %s " % (APP, VER))
 		l = gtk.Label("\n"+
 					  " Copyright(C) 2001-'04, Tamito KAJIYAMA \n"+
 					  " Copyright(C) 2003-'04, Atzm WATANABE \n")
@@ -923,7 +1035,7 @@ class Application:
 		frame.show()
 		self.about.vbox.add(frame)
 
-		b = gtk.Button(unicode(_("OK"), "utf8"))
+		b = gtk.Button(unicode(_("OK"), "utf8"), gtk.STOCK_OK)
 		b.connect("clicked", aboutdestroy, None)
 		b.show()
 		self.about.action_area.pack_start(b, gtk.TRUE, gtk.TRUE, 0)
@@ -1045,8 +1157,8 @@ class ChannelDialog:
 
 		bbox = gtk.HBox(gtk.FALSE, 1)
 		bbox.set_border_width(2)
-		ok = gtk.Button(unicode(_("OK"), "utf8"))
-		ca = gtk.Button(unicode(_("Cancel"), "utf8"))
+		ok = gtk.Button(unicode(_("OK"), "utf8"), gtk.STOCK_OK)
+		ca = gtk.Button(unicode(_("Cancel"), "utf8"), gtk.STOCK_CANCEL)
 		ok.connect("clicked", self.ok, None)
 		ca.connect("clicked", self.cancel, None)
 		bbox.pack_start(ok, gtk.TRUE, gtk.TRUE, 2)
@@ -1378,6 +1490,7 @@ class LogWindow:
 	def __init__(self, app, logger):
 		self.app = app
 		self.logger = logger
+		self.parser = Parser('loose')
 
 		self.labels = {
 			LogManager.ALL_CHANNELS: unicode(_("All"), "utf-8"),
@@ -1474,8 +1587,7 @@ class LogWindow:
 		self.window.set_title(unicode(_("GBottler"), "utf-8") + ":" + \
 							  unicode(_("Logs"), "utf-8"))
 
-		self.main_vbox = gtk.VBox(gtk.FALSE, 1)
-		self.main_vbox.set_border_width(1)
+		self.main_vbox = gtk.VBox(gtk.FALSE, 0)
 		self.window.add(self.main_vbox)
 		self.main_vbox.show()
 
@@ -1498,37 +1610,71 @@ class LogWindow:
 		# End of creating Menubar
 
 		self.hbox = gtk.HBox()
-		self.pb = gtk.Button(unicode(_("Play"), "utf-8"))
+		pb_h = self.app.get_imglabelbox_with_stock(gtk.STOCK_REFRESH,
+												   gtk.ICON_SIZE_BUTTON,
+												   unicode(_("Play"), "utf-8"))
+		self.pb = gtk.Button()
+		self.pb.add(pb_h)
 		self.pb.connect("clicked", self.play, None)
+		pb_h.show()
 		self.pb.show()
 		self.hbox.pack_start(self.pb, gtk.FALSE, gtk.TRUE, 0)
 
-		self.vb = gtk.Button(unicode(_("Vote"), "utf-8"))
+		vb_h = self.app.get_imglabelbox_with_stock(gtk.STOCK_YES,
+												   gtk.ICON_SIZE_BUTTON,
+												   unicode(_("Vote"), "utf-8"))
+		self.vb = gtk.Button()
+		self.vb.add(vb_h)
 		self.vb.connect("clicked", self.vote, None)
+		vb_h.show()
 		self.vb.set_sensitive(gtk.FALSE)
 		self.vb.show()
 		self.hbox.pack_start(self.vb, gtk.FALSE, gtk.TRUE, 0)
 
-		self.ab = gtk.Button(unicode(_("Agree"), "utf-8"))
+		ab_h = self.app.get_imglabelbox_with_stock(gtk.STOCK_NO,
+												   gtk.ICON_SIZE_BUTTON,
+												   unicode(_("Agree"), "utf-8"))
+		self.ab = gtk.Button()
+		self.ab.add(ab_h)
 		self.ab.connect("clicked", self.agree, None)
+		ab_h.show()
 		self.ab.set_sensitive(gtk.FALSE)
 		self.ab.show()
 		self.hbox.pack_start(self.ab, gtk.FALSE, gtk.TRUE, 0)
 
 		self.l = gtk.Label(" " + unicode(_("Range"), "utf-8") + ": ")
 		self.range_l = gtk.Label(unicode(_("Last 50 bottles"), "utf-8"))
+		self.r_e_box = gtk.EventBox()
+		self.r_e_box.set_events(gtk.gdk.BUTTON_PRESS_MASK)
+		self.r_e_box.add(self.range_l)
 		self.hbox.pack_start(self.l, gtk.FALSE, gtk.TRUE, 0)
-		self.hbox.pack_start(self.range_l, gtk.FALSE, gtk.TRUE, 0)
+		self.hbox.pack_start(self.r_e_box, gtk.FALSE, gtk.TRUE, 0)
 		self.l.show()
 		self.range_l.show()
+		self.r_e_box.show()
+		self.r_e_box.connect("button-press-event", self.button_popup,
+							 self.item_factory.get_widget("/%s/%s" % (self.msg["view"][1],
+																	  self.msg["range"][1])))
+
+		self.m = gtk.Label("    " + unicode(_("Style"), "utf-8") + ": ")
+		self.style_l = gtk.Label(self.msg['talk style'])
+		self.s_e_box = gtk.EventBox()
+		self.s_e_box.set_events(gtk.gdk.BUTTON_PRESS_MASK)
+		self.s_e_box.add(self.style_l)
+		self.hbox.pack_start(self.m, gtk.FALSE, gtk.TRUE, 0)
+		self.hbox.pack_start(self.s_e_box, gtk.FALSE, gtk.TRUE, 0)
+		self.m.show()
+		self.style_l.show()
+		self.s_e_box.show()
+		self.s_e_box.connect("button-press-event", self.button_popup,
+							 self.item_factory.get_widget("/%s/%s" % (self.msg["view"][1],
+																	  self.msg["style"][1])))
 
 		self.main_vbox.pack_start(self.hbox, gtk.FALSE, gtk.TRUE, 0)
 		self.hbox.show()
 
 		self.sw1 = gtk.ScrolledWindow()
 		self.sw1.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-		self.main_vbox.pack_start(self.sw1)
-		self.sw1.show()
 		self.cols = [
 			[unicode(_("Datetime"), "utf-8"), gtk.SORT_DESCENDING],
 			[unicode(_("Ghost"), "utf-8"), gtk.SORT_DESCENDING],
@@ -1546,6 +1692,7 @@ class LogWindow:
 		self.entries.set_column_width(0, 100)
 		self.entries.set_column_auto_resize(1, gtk.TRUE)
 		self.sw1.add(self.entries)
+		self.sw1.show()
 		self.entries.show()
 
 		for i in range(len(self.cols)):
@@ -1567,7 +1714,6 @@ class LogWindow:
 
 		self.sw2 = gtk.ScrolledWindow()
 		self.sw2.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-		self.main_vbox.pack_start(self.sw2)
 		self.sw2.show()
 
 		self.main_text = gtk.TextView()
@@ -1578,31 +1724,90 @@ class LogWindow:
 		self.main_text.show()
 		self.sw2.add(self.main_text)
 
+		self.pane = gtk.VPaned()
+		self.pane.pack1(self.sw1)
+		self.pane.pack2(self.sw2)
+		self.pane.show()
+		self.main_vbox.pack_start(self.pane)
+
 		self.monitor = self.main_text.get_buffer()
+		self.sakuratag = self.monitor.create_tag('sakura')
+		self.sakuratag.set_property('foreground', 'black')
+		self.kerotag = self.monitor.create_tag('kero')
+		self.kerotag.set_property('foreground', 'brown')
+		self.synctag = self.monitor.create_tag('sync')
+		self.synctag.set_property('foreground', 'blue')
+		self.scripttag = self.monitor.create_tag('script')
+		self.scripttag.set_property('foreground', 'dark green')
 
 		# initialize
-		self.pattern = re.compile(r"\\s\[\-?\d+?\]|\\w\d+?|\\_[qs]|\\URL(\[.+?\])?\[.+?\]|\\q\d\[\#\]\[.+?\]|\\n\[half\]|\\[tncyze]")
-		self.splitter = re.compile(r"\\h|\\u|\\0|\\1")
-		self.replacer = re.compile(r"\n\n")
+		self.view_style = self.STYLE_TALK
 		self.current_sort = 0
 		self.initialize(None, self.window)
 
 	def set_style(self, data, widget):
 		if data == self.STYLE_TALK:
-			self.pattern = re.compile(r"\\s\[\-?\d+?\]|\\w\d+?|\\_[qs]|\\URL(\[.+?\])?\[.+?\]|\\q\d\[\#\]\[.+?\]|\\n\[half\]|\\[tncyze]")
-			self.splitter = re.compile(r"\\h|\\u|\\0|\\1")
-			self.replacer = re.compile(r"\n\n")
+			self.style_l.set_text(self.msg['talk style'])
 		elif data == self.STYLE_SCRIPT:
-			self.pattern = re.compile("")
-			self.splitter = re.compile("")
-			self.replacer = re.compile("")
+			self.style_l.set_text(self.msg['script style'])
 		elif data == self.STYLE_SCRIPT_WITH_LINEFEED:
-			self.pattern = re.compile("")
-			self.splitter = re.compile(r"(\\h|\\u|\\0|\\1)")
-			self.replacer = re.compile(r"\n\n")
+			self.style_l.set_text("%s(%s)" % (self.msg['script style'], self.msg['linefeed']))
 
+		self.view_style = data
 		self.select(None, self.entries.selection[0], None, None)
 
+	def select(self, clist, row, col, event):
+		current = 'sakura'
+		self.monitor.set_text("")
+		for chunk in self.parser.parse(unicode(self.entries.get_text(row, 5), "utf-8")):
+			if chunk[0] == TEXT_STRING:
+				if chunk[1][0][0] == SCRIPT_TEXT:
+					self.monitor.insert_with_tags_by_name(self.monitor.get_end_iter(),
+														  chunk[1][0][1], current)
+
+			elif chunk[0] == TEXT_META:
+				name, args = chunk[1], chunk[2:]
+
+				if name in [r'\0', r'\h']:
+					current = 'sakura'
+				elif name in [r'\1', r'\u']:
+					current = 'kero'
+				elif name == r'\_s':
+					current = 'sync'
+
+				if self.view_style == self.STYLE_TALK:
+					pre_end = self.monitor.get_end_iter()
+					first = pre_end.backward_char()
+					if name in [r'\0', r'\h', r'\1', r'\u', r'\_s'] and \
+						   pre_end.get_char() != '\n' and first:
+						self.monitor.insert_with_tags_by_name(self.monitor.get_end_iter(),
+															  '\n', 'script')
+
+					elif name == r'\URL':
+						for n in range(len(args)):
+							self.monitor.insert_with_tags_by_name(self.monitor.get_end_iter(),
+																  "[%s]" % args[n][0][1], 'script')
+
+				elif self.view_style == self.STYLE_SCRIPT:
+					self.monitor.insert_with_tags_by_name(self.monitor.get_end_iter(),
+														  name, 'script')
+					for n in range(len(args)):
+						self.monitor.insert_with_tags_by_name(self.monitor.get_end_iter(),
+															  "[%s]" % args[n], 'script')
+
+				elif self.view_style == self.STYLE_SCRIPT_WITH_LINEFEED:
+					self.monitor.insert_with_tags_by_name(self.monitor.get_end_iter(),
+														  name, 'script')
+					for n in range(len(args)):
+						self.monitor.insert_with_tags_by_name(self.monitor.get_end_iter(),
+															  "[%s]" % args[n], 'script')
+					if name == r'\n':
+						self.monitor.insert_with_tags_by_name(self.monitor.get_end_iter(),
+															  '\n', 'script')
+
+		self.monitor.insert_with_tags_by_name(self.monitor.get_end_iter(),
+											  '\n', 'script')
+
 	def sort(self, widget, colnum):
 		if type(widget) is gtk.Button:
 			self.cols[colnum][1] = not self.cols[colnum][1]
@@ -1619,23 +1824,16 @@ class LogWindow:
 			self.entries.moveto(self.entries.selection[0], colnum, 0.5, 0)
 			self.set_focus_row(self.entries.selection[0], self.entries.rows)
 
-	def select(self, clist, row, col, event):
-		#self.entries.moveto(row, 0, 0.5, 0)
-		script = unicode(self.entries.get_text(row, 5), "utf-8")
-
-		# Bad /-P
-		script  = self.pattern.sub("", script)
-		script  = self.splitter.split(script)
-		script  = string.join(script, "\n")
-		script  = self.replacer.sub("", script)
-		self.monitor.set_text(script)
-
 	def popup(self, widget, event):
 		if event.button == 3:
 			sel = self.entries.get_selection_info(int(event.x), int(event.y))
 			self.set_focus_row(sel[0], self.entries.rows)
 			self.popup_menu.popup(None, None, None, event.button, 0)
 
+	def button_popup(self, widget, event, sender):
+		if event.button in [1, 3]:
+			sender.popup(None, None, None, event.button, 0)
+
 	def open(self, widget, data):
 		self.window.show()
 
@@ -1780,4 +1978,3 @@ class LogWindow:
 if __name__ == "__main__":
 	app = Application()
 	app.main()
-
